/*
Gtk+ogltk - Gtk+ OpenGL Toolkit

Copyright (C) 2003 SandorLabs

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Contact :
    John Kiss - jkiss@sandorapplications.com


*/
#if !defined(OGLTK_GLOBJECT_H)
#define OGLTK_GLOBJECT_H
#include "ogltk/ogltk.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define	OGLTK_TYPE_GLOBJECT		            ( ogltk_globject_get_type() )
#define OGLTK_GLOBJECT(object)		        (OGLTK_CHECK_CAST ((object),      OGLTK_TYPE_GLOBJECT, GLobject))
#define OGLTK_GLOBJECT_CLASS(klass)		    (OGLTK_CHECK_CLASS_CAST ((klass), OGLTK_TYPE_GLOBJECT, GLobject_class))
#define OGLTK_IS_GLOBJECT(object)		    (OGLTK_CHECK_TYPE ((object),      OGLTK_TYPE_GLOBJECT))
#define OGLTK_IS_GLOBJECT_CLASS(klass)	    (OGLTK_CHECK_CLASS_TYPE ((klass), OGLTK_TYPE_GLOBJECT))
#define	OGLTK_GLOBJECT_GET_CLASS(object)    (OGLTK_CHECK_GET_CLASS ((object), OGLTK_TYPE_GLOBJECT, GLobject_class))

typedef enum
{
    GLOBJECT_TYPE_NONE,
    
    GLOBJECT_TYPE_PRIMITIVES,
    GLOBJECT_TYPE_DISPLAY_LIST,
    GLOBJECT_TYPE_CUSTOM,
    
    GLOBJECT_TYPE_LAST
}GlObjectTypes;

typedef struct _ogltk_globject	            GLobject;
typedef struct _ogltk_globject_class        GLobject_class;


struct _ogltk_globject
{
    GObject             parent_instance;

    /* Properties */
    GlObjectTypes       type;
    gchar               *name;

    gltTranslate        translate;
    gltRotate           rotate;
    
    gboolean            selected;
    
    gltColor            *color;
};

struct _ogltk_globject_class
{
  GObjectClass              parent_class;

  ogltk_callback            draw;  
  ogltk_callback            draw_start;
  ogltk_callback            draw_finish;

  ogltk_callback            translate;
  ogltk_callback            rotate;

  ogltk_callback            selected;
  ogltk_callback            unselected;
  
};

GType                       ogltk_globject_get_type(void);

gboolean                    ogltk_globject_set_type(GLobject *globject, GlObjectTypes type);

gboolean                    ogltk_globject_destroy(GLobject *globject);

/* Cannot Call this get_type, conflicts with class type declaration */
GlObjectTypes               ogltk_globject_type(GLobject *globject);

gboolean                    ogltk_globject_set_name(GLobject *globject, const char *name);
const char*                 ogltk_globject_get_name(GLobject *globject);

gboolean                    ogltk_globject_select(GLobject *globject, gboolean enable);
gboolean                    ogltk_globject_is_selected(GLobject *globject);

gboolean                    ogltk_globject_set_translate_matrix(GLobject *globject, gfloat x, gfloat y, gfloat z);

gboolean                    ogltk_globject_enable_rotate_matrix(GLobject *globject);
gboolean                    ogltk_globject_set_rotate_matrix(GLobject *globject, gfloat angle,gfloat x, gfloat y, gfloat z);
gboolean                    ogltk_globject_set_rotate_matrix_and_enable(GLobject *globject,gfloat angle,gfloat x,gfloat y,gfloat z);
gboolean                    ogltk_globject_disable_rotate_matrix(GLobject *globject);
gboolean                    ogltk_globject_update_rotate_matrix_angle(GLobject *globject,gfloat angle);

const gltTranslate*         ogltk_globject_get_translate_matrix(GLobject *globject);
const gltRotate*            ogltk_globject_get_rotate_matrix(GLobject *globject);

const gltColor*             ogltk_globject_get_color(GLobject *globject);
gboolean                    ogltk_globject_set_color(GLobject *globject, gfloat red, gfloat green, gfloat blue, gfloat alpha);
gboolean                    ogltk_globject_draw_color(GLobject *globject);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* OGLTK_GLOBJECT_H */ 

