/*
Gtk+ogltk - Gtk+ OpenGL Toolkit

Copyright (C) 2003 SandorLabs

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Contact :
    John Kiss - jkiss@sandorapplications.com


*/
#if !defined(OGLTK_LIGHTSOURCE_H)
#define OGLTK_LIGHTSOURCE_H
#include <glib.h>
#include <glib-object.h>
#include "ogltk/ogltk.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define	OGLTK_TYPE_LIGHTSOURCE		            ( ogltk_lightsource_get_type() )
#define OGLTK_LIGHTSOURCE(object)		        (G_TYPE_CHECK_INSTANCE_CAST ((object), OGLTK_TYPE_LIGHTSOURCE, ogltk_lightsource))
#define OGLTK_LIGHTSOURCE_CLASS(klass)		    (G_TYPE_CHECK_CLASS_CAST    ((klass),  OGLTK_TYPE_LIGHTSOURCE, ogltk_lightsource_class))
#define OGLTK_IS_LIGHTSOURCE(object)		    (G_TYPE_CHECK_INSTANCE_TYPE ((object), OGLTK_TYPE_LIGHTSOURCE))
#define OGLTK_IS_LIGHTSOURCE_CLASS(klass)	    (G_TYPE_CHECK_CLASS_TYPE    ((klass),  OGLTK_TYPE_LIGHTSOURCE))
#define	OGLTK_LIGHTSOURCE_GET_CLASS(object)     (G_TYPE_INSTANCE_GET_CLASS  ((object), OGLTK_TYPE_LIGHTSOURCE, ogltk_lightsource_class))

typedef struct _ogltk_lightsource	            ogltk_lightsource;
typedef struct _ogltk_lightsource_class         ogltk_lightsource_class;

typedef enum
{
    /* Reserved for Scene */
    OGLTK_LTSRC_NONE  = 0,
    
    OGLTK_LTSRC_ONE   = GL_LIGHT0,
    OGLTK_LTSRC_TWO   = GL_LIGHT1,
    OGLTK_LTSRC_THREE = GL_LIGHT2,
    OGLTK_LTSRC_FOUR  = GL_LIGHT3,
    
    OGLTK_LTSRC_LAST
}OgltkLightSource;


typedef enum
{
    OGLTK_FLICK_OFF,
    OGLTK_FLICK_ON,

}OgltkLightFlick;


struct _ogltk_lightsource
{
    GObject             parent_instance;
    
    OgltkLightSource    source;
    gltColor            ambient_color;
    gltColor            diffuse_color;
    gltTranslate        location;
    OgltkLightFlick     enabled;
    
    gfloat              actual_location;
};


struct _ogltk_lightsource_class
{
  GObjectClass              parent_class;
  
  ogltk_callback            illuminate;

};

GType                       ogltk_lightsource_get_type(void);

gboolean                    ogltk_lightsource_set_ambient_color(ogltk_lightsource *light, gfloat red, gfloat green, gfloat blue, gfloat alpha);

gboolean                    ogltk_lightsource_set_diffuse_color(ogltk_lightsource *light, gfloat red, gfloat green, gfloat blue, gfloat alpha);

gboolean                    ogltk_lightsource_set_location(ogltk_lightsource *light, gfloat x, gfloat y, gfloat z);

gboolean                    ogltk_lightsource_switch(ogltk_lightsource *light, OgltkLightFlick flick);

OgltkLightFlick             ogltk_lightsource_switch_position(ogltk_lightsource *light);

gboolean                    ogltk_lightsource_illuminate(ogltk_lightsource *light);

gboolean                    ogltk_lightsource_destroy(ogltk_lightsource *light);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* OGLTK_LIGHTSOURCE_H */ 

