/*
Gtk+ogltk - Gtk+ OpenGL Toolkit

Copyright (C) 2003 SandorLabs

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Contact :
    John Kiss - jkiss@sandorapplications.com


*/
#if !defined(OGLTK_SCENE_H)
#define OGLTK_SCENE_H
#include "ogltk/ogltk.h"
#include "ogltk/globject.h"
#include "ogltk/lightsource.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define	OGLTK_TYPE_SCENE		            ( ogltk_scene_get_type() )
#define OGLTK_SCENE(object)		            (OGLTK_CHECK_CAST ((object),      OGLTK_TYPE_SCENE, ogltk_scene))
#define OGLTK_SCENE_CLASS(klass)		    (OGLTK_CHECK_CLASS_CAST ((klass), OGLTK_TYPE_SCENE, ogltk_scene_class))
#define OGLTK_IS_SCENE(object)		        (OGLTK_CHECK_TYPE ((object),      OGLTK_TYPE_SCENE))
#define OGLTK_IS_SCENE_CLASS(klass)	        (OGLTK_CHECK_CLASS_TYPE ((klass), OGLTK_TYPE_SCENE))
#define	OGLTK_SCENE_GET_CLASS(object)	    (OGLTK_CHECK_GET_CLASS ((object), OGLTK_TYPE_SCENE, ogltk_scene_class))

typedef enum
{
    OGLTK_SCENE_DISPLAY_NONE,
    
    OGLTK_SCENE_DISPLAY_GLOBJECT,
    OGLTK_SCENE_DISPLAY_LIST,
    
    OGLTK_SCENE_DISPLAY_LAST

}OgltkSceneDisplayOrder;

typedef struct _ogltk_scene	            ogltk_scene;
typedef struct _ogltk_scene_class       ogltk_scene_class;


struct _ogltk_scene
{
    GObject             parent_instance;

    GList               *globjects;
    GList               *displaylist;
    /* FPS */
    GTimer              *fps_timer;
    gint                fps;
    gint                fps_notify_time;
    
    gboolean            bobject_events;     /* Enable Object Signals before and after each object is displayed */
    
    OgltkSceneDisplayOrder display_order;   /* Set which order objects will be rendered by this scene? display list or globjects */
    
    gint                fps_signal_time;    /* Time in seconds when fps-event signals are sent */

    ogltk_lightsource   *lightsources;      /* FIXME JSK , Should be GLIST of lightsources */
         
};

struct _ogltk_scene_class
{
  GObjectClass              parent_class;
  
  ogltk_callback            render;
  ogltk_callback            render_start;
  ogltk_callback            render_finish;
  ogltk_callback            object_start;
  ogltk_callback            object_finsih;
    
  ogltk_callback            fps_notify;
  
};

GType                       ogltk_scene_get_type (void);

/* Scene Constructor Methods */
ogltk_scene*                ogltk_scene_new(void);


/* Property Control Methods */
gboolean                    ogltk_scene_set_display_order(ogltk_scene *scene, OgltkSceneDisplayOrder order);
OgltkSceneDisplayOrder      ogltk_scene_get_display_order(ogltk_scene *scene);
gboolean                    ogltk_scene_enable_object_events(ogltk_scene *scene, gboolean enable);
gboolean                    ogltk_scene_object_events_enabled(ogltk_scene *scene);

/* globject list control methods */
gboolean                    ogltk_scene_add_globject(ogltk_scene *scene, GLobject *object);

/* Frame Per Second Notification Control */
gboolean                    ogltk_scene_fps_start(ogltk_scene *scene, gint notify_time);
gboolean                    ogltk_scene_fps_stop(ogltk_scene *scene);

gboolean                    ogltk_scene_clear(ogltk_scene *scene);


GLobject*                   ogltk_scene_select(ogltk_scene *scene, gfloat x, gfloat y);

gboolean                    ogltk_scene_add_lightsource(ogltk_scene *scene, ogltk_lightsource *light);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OGLTK_SCENE_H */ 

