/*
Gtk+ogltk - Gtk+ OpenGL Toolkit

Copyright (C) 2003 SandorLabs

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Contact :
    John Kiss - jkiss@sandorapplications.com


*/
#if !defined(GTK_OGLTK_VIEW_H)
#define GTK_OGLTK_VIEW_H
#include <gdk/gdkglconfig.h>

#include "ogltk/ogltk.h"
#include "ogltk/scene.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define	OGLTK_TYPE_VIEW			            ( ogltk_view_get_type() )
#define OGLTK_VIEW(object)		            (OGLTK_CHECK_CAST ((object),      OGLTK_TYPE_VIEW, ogltk_view))
#define OGLTK_VIEW_CLASS(klass)		        (OGLTK_CHECK_CLASS_CAST ((klass), OGLTK_TYPE_VIEW, ogltk_view_class))
#define OGLTK_IS_VIEW(object)		        (OGLTK_CHECK_TYPE ((object),      OGLTK_TYPE_VIEW))
#define OGLTK_IS_VIEW_CLASS(klass)	        (OGLTK_CHECK_CLASS_TYPE ((klass), OGLTK_TYPE_VIEW))
#define	OGLTK_VIEW_GET_CLASS(object)	    (OGLTK_CHECK_GET_CLASS ((object), OGLTK_TYPE_VIEW, ogltk_view_class))

typedef struct _ogltk_view	            ogltk_view;
typedef struct _ogltk_view_class        ogltk_view_class;


typedef enum
{
    OGLTK_CONFIG_RGB          = GDK_GL_MODE_RGB,        /* 0 */
    OGLTK_CONFIG_RGBA         = GDK_GL_MODE_RGBA,       /* same as RGB */
    OGLTK_CONFIG_INDEX        = GDK_GL_MODE_INDEX,      
    OGLTK_CONFIG_SINGLE       = GDK_GL_MODE_SINGLE,
    OGLTK_CONFIG_DOUBLE       = GDK_GL_MODE_DOUBLE,
    OGLTK_CONFIG_STERO        = GDK_GL_MODE_STEREO,
    OGLTK_CONFIG_ALPHA        = GDK_GL_MODE_ALPHA,
    OGLTK_CONFIG_DEPTH        = GDK_GL_MODE_DEPTH,
    OGLTK_CONFIG_STENCIL      = GDK_GL_MODE_STENCIL,
    OGLTK_CONFIG_ACCUM        = GDK_GL_MODE_ACCUM,
    OGLTK_CONFIG_MULTISAMPLE  = GDK_GL_MODE_MULTISAMPLE

} OGLTKConfigMode;

typedef enum
{
    OGLTK_RENDER_RGBA       = GDK_GL_RGBA_TYPE,
    OGLTK_RENDER_INDEX      = GDK_GL_COLOR_INDEX_TYPE
    
} OGLTKRenderType;


typedef enum
{
    OGLTK_FACE_FRONT            = GL_FRONT,
    OGLTK_FACE_BACK             = GL_BACK,
    OGLTK_FACE_FRONT_AND_BACK   = GL_FRONT_AND_BACK,

}OgltkFace;

typedef enum
{
    OGLTK_MAT_NONE                  = 0,
    OGLTK_MAT_AMBIENT               = GL_AMBIENT,
    OGLTK_MAT_DIFFUSE               = GL_DIFFUSE,
    OGLTK_MAT_SPECULAR              = GL_SPECULAR,
    OGLTK_MAT_EMISION               = GL_EMISSION,
    OGLTK_MAT_AMBIENT_AND_DIFFUSE   = GL_AMBIENT_AND_DIFFUSE,
    OGLTK_MAT_LAST                  = (OGLTK_MAT_AMBIENT_AND_DIFFUSE + 1),

}OgltkMaterialMode;


struct _ogltk_view
{
    GtkDrawingArea      parent_instance;
    
    ogltk_scene         *scene;
    
    OGLTKConfigMode     config_mode;
    OGLTKRenderType     render_mode;
    gboolean            direct;         /* Should use direct connection to graphics system */
    
    gltTranslate        translate;
    gltRotate           rotate;
    gltColor            background;
    
    gboolean            light_enabled;
    OgltkFace           light_polygon_faces;
    OgltkMaterialMode   light_material;
    
};


struct _ogltk_view_class
{
  GtkDrawingAreaClass       parent_class;

};

GType                       ogltk_view_get_type (void);

gint                        ogltk_view_get_width(ogltk_view *view);
gint                        ogltk_view_get_height(ogltk_view *view);

gboolean                    ogltk_view_set_translate_matrix(ogltk_view *view, gfloat x, gfloat y, gfloat z);
gboolean                    ogltk_view_enable_rotate_matrix(ogltk_view *view);
gboolean                    ogltk_view_set_rotate_matrix(ogltk_view *view, gfloat angle,gfloat x, gfloat y, gfloat z);
gboolean                    ogltk_view_set_rotate_matrix_and_enable(ogltk_view *view,gfloat angle,gfloat x,gfloat y,gfloat z);
gboolean                    ogltk_view_disable_rotate_matrix(ogltk_view *view);
gboolean                    ogltk_view_update_rotate_matrix_angle(ogltk_view *view,gfloat angle);

const gltTranslate*         ogltk_view_get_translate_matrix(ogltk_view *view);
const gltRotate*            ogltk_view_get_rotate_matrix(ogltk_view *view);

gboolean                    ogltk_view_set_config_mode(ogltk_view *view, OGLTKConfigMode config);
gboolean                    ogltk_view_enable_direct(ogltk_view *view, gboolean enable);
gboolean                    ogltk_view_set_render_type(ogltk_view *view, OGLTKRenderType config);
gboolean                    ogltk_view_update_capability(ogltk_view *view);

gboolean                    ogltk_view_draw_gl_begin(ogltk_view *view);
gboolean                    ogltk_view_draw_gl_end(ogltk_view *view);
gboolean                    ogltk_view_swap_buffers(ogltk_view *view);

gboolean                    ogltk_view_set_scene(ogltk_view *view, ogltk_scene *scene); /* DEPRECIATED */

gboolean                    ogltk_view_bind_scene(ogltk_view *view, ogltk_scene *scene);
ogltk_scene*                ogltk_view_get_scene(ogltk_view *view);
gboolean                    ogltk_view_unbind_scene(ogltk_view *view);
gboolean                    ogltk_view_scene_is_bound(ogltk_view *view);

gboolean                    ogltk_view_render(ogltk_view *view);

gboolean                    ogltk_view_set_background_color(ogltk_view *view, gfloat red, gfloat green, gfloat blue, gfloat alpha);

gboolean                    ogltk_view_enable_lighting(ogltk_view *view, gboolean enable);
gboolean                    ogltk_view_lighting_enabled(ogltk_view *view);
gboolean                    ogltk_view_set_color_material(ogltk_view *view, OgltkFace face, OgltkMaterialMode mode);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* GTK_OGLTK_VIEW_H */
