/*    
    Example box + Gtk+ogltk - Gtk+ OpenGL Toolkit 
    
    Copyright (C) 2003 SandorLabs

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact :
        John Kiss - jkiss@sandorapplications.com

*/
#include <gtk/gtk.h>
#include <ogltk/ogltk.h>
#include <ogltk/view3D.h>
#include <ogltk/scene.h>
#include <ogltk/lightbulb.h>
#include <ogltk/glo_box.h>

static ogltk_scene *scene          = NULL;
static GLobject    *glbox = NULL;
static guint idle_id = 0;

static gboolean idle (GtkWidget *widget)
{
  
  ogltk_globject_update_rotate_matrix_angle(glbox,(glbox->rotate.angle + 0.02));
  
  gtk_widget_queue_draw (widget);

  return TRUE;
}


static void idle_add (GtkWidget *widget)
{
  if (idle_id == 0)
    {
      idle_id = gtk_idle_add_priority (GDK_PRIORITY_REDRAW,
				       (GtkFunction) idle,
				       widget);
    }
}

static GtkWidget* init_ogltk(void)
{
    GtkWidget *retval = NULL;
    
    /* Create the 3D View Window */
    retval = ogltk_view3D_new(  TRUE,
                            (OGLTK_CONFIG_RGB | OGLTK_CONFIG_DEPTH | OGLTK_CONFIG_DOUBLE),
                            OGLTK_RENDER_RGBA,5.0,100.0);
 
    /* Did it get created ? */
    if (OGLTK_IS_VIEW(retval) == TRUE)
    {
        
        /* Enable Lighting for entire view */
        ogltk_view_enable_lighting(OGLTK_VIEW(retval), TRUE);
        
        /* Set Material for lite objects */
        ogltk_view_set_color_material(OGLTK_VIEW(retval),OGLTK_FACE_FRONT,OGLTK_MAT_AMBIENT_AND_DIFFUSE);
        
        /* Every 3D World needs a scene */
        scene = ogltk_scene_new();
        
        /* Did it get created ? */
        if (OGLTK_IS_SCENE(scene) == TRUE)
        {
            
            /* Bind the Scene to the view */
            if (ogltk_view_set_scene(OGLTK_VIEW(retval),scene) == TRUE)
            {
                ogltk_lightsource *lightbulb = NULL;
                
                /* Need a light source, how about a simple little lightbulb 
                    put it way up(5.0) and to the right(5.0) and toward the view(10) */
                lightbulb = ogltk_lightbulb_new(OGLTK_LTSRC_ONE,5.0,5.0,10.0);
                
                if (OGLTK_IS_LIGHTBULB(lightbulb) == TRUE)
                {
                    
                    /* Turn the lightbulb on */
                    ogltk_lightsource_switch(lightbulb,OGLTK_FLICK_ON);
                    
                    /* Set ambient and diffuse light color */
                    ogltk_lightsource_set_ambient_color(lightbulb,0.1,0.1,0.1,1.0);
                    ogltk_lightsource_set_diffuse_color(lightbulb,1.0,1.0,1.0,1.0);
                    
                    
                    /* Add the lightbulb to the scene */
                    ogltk_scene_add_lightsource(scene,lightbulb);
                }
                
            }
            else
            {
                g_warning("Failed to bind view with scene");
                
                gtk_widget_destroy(retval);
                
                retval = NULL;
            }
            
        }
    
    }    

    return(retval);
}

           
int main(int   argc, char *argv[])
{
    GtkWidget *window = NULL;
    GtkWidget *vbox = NULL;
    GtkWidget *view = NULL;

    /* Initialize GTK */
    gtk_init (&argc, &argv);
    
    /* Initialize GL (Needed by Gtkglext */
    gtk_gl_init (&argc, &argv);

    /* Create your Application Window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (window), "Gtk+-OGLTK Box");

    /* ogltk, create the view and a scene */
    view = init_ogltk();
    
    
    if (OGLTK_IS_VIEW(view) == TRUE)
    {
        vbox = gtk_vbox_new (FALSE, 0);
        gtk_container_add (GTK_CONTAINER (window), vbox);
        gtk_widget_show (vbox);
        
        gtk_box_pack_start (GTK_BOX (vbox), view, TRUE, TRUE, 0);
        
        gtk_widget_show (view);
    
        g_signal_connect (G_OBJECT (window), "delete_event",
                G_CALLBACK (gtk_main_quit), NULL);
    
        glbox =  glo_box_new(5,7,2);
        ogltk_scene_add_globject(scene, glbox);
        ogltk_globject_set_translate_matrix(glbox,0.0,0.0,0.0);
        ogltk_globject_set_rotate_matrix_and_enable(glbox,0.0,1.0,1.0,1.0);
        ogltk_globject_set_color(glbox,1.0,1.0,1.0,1.0);
     
        gtk_widget_show (window);
        idle_add (view);
       
        gtk_main ();
        
        g_object_unref(scene);
    }
        
    return(0);
}
