/*    
    Example gears + Gtk+ogltk - Gtk+ OpenGL Toolkit 
    
    Copyright (C) 2003 SandorLabs

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact :
        John Kiss - jkiss@sandorapplications.com

*/
#include <gtk/gtk.h>
#include <ogltk/ogltk.h>
#include <ogltk/view3D.h>
#include <ogltk/scene.h>
#include <ogltk/glo_gear.h>
#include <ogltk/lightbulb.h>

static ogltk_scene *scene          = NULL;
static guint idle_id = 0;
GLobject  *gear1 = NULL;
GLobject  *gear2 = NULL;
GLobject  *gear3 = NULL;
gfloat    angle = 0.0f;

static gboolean idle (GtkWidget *widget)
{
  
  angle += 0.50;
  
  /* update each GLobjects rotation matrix */
  ogltk_globject_update_rotate_matrix_angle(gear1,(gear1->rotate.angle = angle));
  ogltk_globject_update_rotate_matrix_angle(gear2,(gear2->rotate.angle = angle * -2.0 - 9));
  ogltk_globject_update_rotate_matrix_angle(gear3,(gear3->rotate.angle = angle * -2.0 - 25));
  
  /* Call View Render which calls render scene */
  gtk_widget_queue_draw (widget);

  return TRUE;
}


static void idle_add (GtkWidget *widget)
{
  if (idle_id == 0)
    {
      idle_id = gtk_idle_add_priority (GDK_PRIORITY_REDRAW,
				       (GtkFunction) idle,
				       widget);
    }
}

static GtkWidget* init_ogltk(void)
{
    GtkWidget *retval = NULL;
    
    /* Create the 3D View Window */
    retval = ogltk_view3D_new(  TRUE,
                            (OGLTK_CONFIG_RGB | OGLTK_CONFIG_DEPTH | OGLTK_CONFIG_DOUBLE),
                            OGLTK_RENDER_RGBA,5.0,100.0);
 
    /* Did it get created ? */
    if (OGLTK_IS_VIEW(retval) == TRUE)
    {
        
        /* Enable Lighting for entire view */
        ogltk_view_enable_lighting(OGLTK_VIEW(retval), TRUE);
        
        /* Set Material for lite objects */
        ogltk_view_set_color_material(OGLTK_VIEW(retval),OGLTK_FACE_FRONT,OGLTK_MAT_AMBIENT_AND_DIFFUSE);
        
        /* Every 3D World needs a scene */
        scene = ogltk_scene_new();
        
        /* Did it get created ? */
        if (OGLTK_IS_SCENE(scene) == TRUE)
        {
            
            /* Bind the Scene to the view */
            if (ogltk_view_set_scene(OGLTK_VIEW(retval),scene) == TRUE)
            {
                ogltk_lightsource *lightbulb = NULL;
                
                /* Need a light source, how about a simple little lightbulb 
                    put it way up(5.0) and to the right(5.0) and toward the view(10) */
                lightbulb = ogltk_lightbulb_new(OGLTK_LTSRC_ONE,5.0,5.0,10.0);
                
                if (OGLTK_IS_LIGHTBULB(lightbulb) == TRUE)
                {
                    
                    /* Turn the lightbulb on */
                    ogltk_lightsource_switch(lightbulb,OGLTK_FLICK_ON);
                    
                    /* Set ambient and diffuse light color */
                    ogltk_lightsource_set_ambient_color(lightbulb,0.1,0.1,0.1,1.0);
                    ogltk_lightsource_set_diffuse_color(lightbulb,1.0,1.0,1.0,1.0);
                    
                    /* Add the lightbulb to the scene */
                    ogltk_scene_add_lightsource(scene,lightbulb);
                }
                
            }
            else
            {
                g_warning("Failed to bind view with scene");
                
                gtk_widget_destroy(retval);
                
                retval = NULL;
            }
            
        }
    
    }    

    return(retval);
}

static void scene_fps_notify(ogltk_scene *scene, gdouble fps, gpointer data)
{
    if (OGLTK_IS_SCENE(scene) == TRUE)
    {
        g_print("FPS = %2.5G\r",fps);
    }
}
          
int main(int   argc, char *argv[])
{
    GtkWidget *window = NULL;
    GtkWidget *vbox = NULL;
    GtkWidget *view = NULL;
    
    /* Initialize GTK */
    gtk_init (&argc, &argv);
    
    /* Initialize GL (Needed by Gtkglext */
    gtk_gl_init (&argc, &argv);

    /* Create your Application Window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (window), "Gtk+-OGLTK Box");

    /* ogltk, create the view and a scene */
    view = init_ogltk();
    
    /* Did we create a view ? */
    if (OGLTK_IS_VIEW(view) == TRUE)
    {
        vbox = gtk_vbox_new (FALSE, 0);
        gtk_container_add (GTK_CONTAINER (window), vbox);
        gtk_widget_show (vbox);
        
        /* Place the view into the Gtk Window */
        gtk_box_pack_start (GTK_BOX (vbox), view, TRUE, TRUE, 0);
        
        /* Show the view */
        gtk_widget_show (view);
    
        g_signal_connect (G_OBJECT (window), "delete_event",
                G_CALLBACK (gtk_main_quit), NULL);

        /* 
            Create a couple of gears and put them in the scene 
            at different locations
        */
        gear1 =  glo_gear_new(1.0, 4.0, 1.0, 20, 0.7);
        ogltk_scene_add_globject(scene, gear1);
        ogltk_globject_set_translate_matrix(gear1,-3.0, -2.0, 0.0);
        ogltk_globject_set_rotate_matrix_and_enable(gear1,0.0,0.0,0.0,1.0);
        ogltk_globject_set_color(gear1,1.0,0.0,0.0,1.0);

        gear2 =  glo_gear_new(0.5, 2.0, 2.0, 10, 0.7);
        ogltk_scene_add_globject(scene, gear2);
        ogltk_globject_set_translate_matrix(gear2,3.1, -2.0, 0.0);
        ogltk_globject_set_rotate_matrix_and_enable(gear2,0.0,0.0,0.0,1.0);
        ogltk_globject_set_color(gear2,0.0,0.0,1.0,1.0);

        gear3 =  glo_gear_new(1.3, 2.0, 0.5, 10, 0.7);
        ogltk_scene_add_globject(scene, gear3);
        ogltk_globject_set_translate_matrix(gear3,-3.1, 4.2, 0.0);
        ogltk_globject_set_rotate_matrix_and_enable(gear3,0.0,0.0,0.0,1.0);
        ogltk_globject_set_color(gear3,0.0,1.0,0.0,1.0);

        /* For giggles, you can ask the scene to send you a signal at intervals you
           specify to get the Frames per second, This signal is event driven by (not by a timer)
           the scene render handler */
        g_signal_connect(scene,"fps-notify",G_CALLBACK(scene_fps_notify),NULL);
        /* Signal every two seconds */
        ogltk_scene_fps_start(scene, 2);

        gtk_widget_show (window);
        idle_add (view);
       
        gtk_main ();
    
        g_object_unref(scene); /* Should destroy all GLobject in the scene */
    }
 
    return(0);
}
